﻿// -----------------------------------------------------------------------
// <copyright file="AppBootstrapper.cs" company="">
//
// The MIT License (MIT)
// 
// Copyright (c) 2012 Christoph Gattnar
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
// documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of
// the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
// BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.
//
// </copyright>
// -----------------------------------------------------------------------

namespace Gat.Controls.TestApplication.Shell
{
	using System;
	using System.Collections.Generic;
	using System.ComponentModel.Composition;
	using System.ComponentModel.Composition.Hosting;
	using System.ComponentModel.Composition.Primitives;
	using System.Linq;
	using System.Reflection;
	using Caliburn.Micro;

	public class AppBootstrapper : Bootstrapper<IShell>
	{
		private CompositionContainer _Container;

		/// <summary>
		/// Configure the Managed Extensibility Framework (MEF) to use.
		/// </summary>
		protected override void Configure()
		{
			// Creating an AggregateCatalog and populating it with AssemblyCatalogs; one for each Assembly in AssemblySource.Instance.
			//
			// ========================================================================
			// AssemblySoure.Instance is the place that Caliburn.Micro looks for Views.
			// ========================================================================
			//
			// Assemblies can add to this at any time during application to make them available to the framework.
			// That can also be done in the Bootstrapper by overriding ****SelectAssemblies()****. By default, the base class returns the
			// assembly that the Application exists in.  So, if all views are in the same assembly as the application, there is no
			// need to worry about this. If there are multiple referenced assemblies that contain views, this is an extension point.
			// Also, if dynamically loading modules, you’ll need to make sure they get registered with your IoC container and
			// the AssemblySoure.Instance when they are loaded. 

			var composablePartCatalog = AssemblySource.Instance.Select(x => new AssemblyCatalog(x)).OfType<ComposablePartCatalog>();
			var catalog = new AggregateCatalog(composablePartCatalog);

			// Create the CompositionContainer with all parts in the catalog (links Exports and Imports)
			_Container = new CompositionContainer(catalog);

			// Provides adding and removing parts at runtime
			var batch = new CompositionBatch();

			// Add Caliburn.Micro-specific services. Default implementations of interfaces..
			batch.AddExportedValue<IWindowManager>(new WindowManager());
			batch.AddExportedValue<IEventAggregator>(new EventAggregator());

			_Container.Compose(batch);
		}

		#region HowToUseContainers

		protected override object GetInstance(Type serviceType, string key)
		{
			string contract = string.IsNullOrEmpty(key) ? AttributedModelServices.GetContractName(serviceType) : key;
			var exports = _Container.GetExportedValues<object>(contract);

			if(exports.Count() > 0)
				return exports.First();

			throw new Exception(string.Format("Could not locate any instances of contract {0}.", contract));
		}

		protected override IEnumerable<object> GetAllInstances(Type serviceType)
		{
			return _Container.GetExportedValues<object>(AttributedModelServices.GetContractName(serviceType));
		}

		// Supply property dependencies to instances of IResult that are executed by the framework (optionally).
		protected override void BuildUp(object instance)
		{
			_Container.SatisfyImportsOnce(instance);
		}

		#endregion

		protected override IEnumerable<Assembly> SelectAssemblies()
		{
			IList<Assembly> assemblies = new List<Assembly>();
			assemblies.Add(Assembly.GetExecutingAssembly());
			assemblies.Add(Assembly.GetAssembly(typeof(AboutControlViewModel)));

			return assemblies;
		}
	}
}
